#include <bnb/glsl.frag>

BNB_IN(0) vec2 var_uv;
BNB_IN(1) vec2 var_bg_mask_uv;


BNB_DECLARE_SAMPLER_2D(0, 1, s_bg);
BNB_DECLARE_SAMPLER_2D(2, 3, s_segmentation_mask);
BNB_DECLARE_SAMPLER_2D(4, 5, bg_video);
BNB_DECLARE_SAMPLER_2D(6, 7, bg_texture);

vec4 cubic(float v)
{
    vec4 n = vec4(1.0, 2.0, 3.0, 4.0) - v;
    vec4 s = n * n * n;
    float x = s.x;
    float y = s.y - 4.0 * s.x;
    float z = s.z - 4.0 * s.y + 6.0 * s.x;
    float w = 6.0 - x - y - z;
    return vec4(x, y, z, w) * (1.0 / 6.0);
}

vec2 rgb_hs(vec3 rgb)
{
    float cmax = max(rgb.r, max(rgb.g, rgb.b));
    float cmin = min(rgb.r, min(rgb.g, rgb.b));
    float delta = cmax - cmin;
    vec2 hs = vec2(0.0);

    if (cmax > cmin) {
        hs.y = delta/cmax;
        if (rgb.r == cmax)
        hs.x = (rgb.g - rgb.b) / delta;
        else
        {
            if (rgb.g == cmax)
            hs.x = 2.0 + (rgb.b - rgb.r) / delta;
            else
            hs.x = 4.0 + (rgb.r - rgb.g) / delta;
        }
        hs.x = fract(hs.x / 6.0);
    }

    return hs;
}


float rgb_v(vec3 rgb)
{
    return max(rgb.r, max(rgb.g, rgb.b));
}


vec3 hsv_rgb(float h, float s, float v)
{
    return v * mix(vec3(1.0), clamp(abs(fract(vec3(1.0, 2.0 / 3.0, 1.0 / 3.0) + h) * 6.0 - 3.0) - 1.0, 0.0, 1.0), s);
}


vec2 scale_uv(vec2 uv, BNB_DECLARE_SAMPLER_2D_ARGUMENT(sampler))
{
    vec2 textureSize = tex_size.xy;
    float aspect_ratio = bnb_SCREEN.y / bnb_SCREEN.x;
    float texture_aspect_ratio = textureSize.y / textureSize.x;
    
    return vec2(mat3(texture_aspect_ratio / aspect_ratio, 0., 0., 0., 1.0, 0., 0.5, 0.5, 1.0) * vec3(uv - 0.5, 1.));
}


void main()
{
    const float threshold = 0.2;
    vec2 bg_uv = var_bg_mask_uv;

    vec2 texSize = background_nn_meta.xy;
    vec2 invTexSize = 1.0 / texSize;

    vec2 texCoords = bg_uv * texSize - 0.5;

    vec2 fxy = fract(texCoords);
    texCoords -= fxy;

    vec4 xcubic = cubic(fxy.x);
    vec4 ycubic = cubic(fxy.y);

    vec4 c = texCoords.xxyy + vec2(-0.5, +1.5).xyxy;

    vec4 s = vec4(xcubic.xz + xcubic.yw, ycubic.xz + ycubic.yw);
    vec4 offset = c + vec4(xcubic.yw, ycubic.yw) / s;

    offset *= invTexSize.xxyy;

    vec4 sample0 = BNB_TEXTURE_2D(BNB_SAMPLER_2D(s_segmentation_mask), offset.xz);
    vec4 sample1 = BNB_TEXTURE_2D(BNB_SAMPLER_2D(s_segmentation_mask), offset.yz);
    vec4 sample2 = BNB_TEXTURE_2D(BNB_SAMPLER_2D(s_segmentation_mask), offset.xw);
    vec4 sample3 = BNB_TEXTURE_2D(BNB_SAMPLER_2D(s_segmentation_mask), offset.yw);

    float sx = s.x / (s.x + s.y);
    float sy = s.z / (s.z + s.w);

    vec4 filtered_mask = mix(mix(sample3, sample2, sx), mix(sample1, sample0, sx), sy);

    float mask = max((filtered_mask.x - threshold) / (1.0 - threshold), 0.0);
    vec3 bg_color = BNB_TEXTURE_2D(BNB_SAMPLER_2D(s_bg), var_uv).rgb;
    vec4 bg_tex_color;
    vec2 uv = var_uv;

    if (bg_mode.x == 0.){
        uv = scale_uv(uv,BNB_PASS_SAMPLER_ARGUMENT(bg_texture));
        #ifndef BNB_VK_1
            uv.y = 1. - uv.y;
        #endif
        vec2 uv_coeff = step(vec2(0.0, 0.0), uv) - step(vec2(1., 1.), uv);
        bg_tex_color = BNB_TEXTURE_2D(BNB_SAMPLER_2D(bg_texture), uv);
        bg_tex_color.rgb *= uv_coeff.x * uv_coeff.y;
    } else {
        uv = scale_uv(uv,BNB_PASS_SAMPLER_ARGUMENT(bg_video));
        #ifndef BNB_VK_1
            uv.y = 1. - uv.y;
        #endif
        vec2 uv_coeff = step(vec2(0.0, 0.0), uv) - step(vec2(1., 1.), uv);
        bg_tex_color = BNB_TEXTURE_2D(BNB_SAMPLER_2D(bg_video), uv);
        bg_tex_color.rgb *= uv_coeff.x * uv_coeff.y;
    }

    if (bg_mode.y == 0.) bg_tex_color = vec4(0.);
    
    bnb_FragColor = vec4(mix(bg_color, bg_tex_color.rgb, mask * bg_tex_color.a), 1.);
}
